@@ -73,7 +73,9 @@
 /* ctype entries */
 #define CTYPE_CONTROL		0x0
 #define CTYPE_STATUS		0x1
+#define CTYPE_NOT_IMPLEMENTED	0x8
 #define CTYPE_ACCEPTED		0x9
+#define CTYPE_REJECTED		0xA
 #define CTYPE_STABLE		0xC
 
 /* opcodes */
@@ -102,12 +104,6 @@ static DBusConnection *connection = NULL;
 static GSList *servers = NULL;
 static GSList *sessions = NULL;
 
-typedef enum {
-	AVCTP_STATE_DISCONNECTED = 0,
-	AVCTP_STATE_CONNECTING,
-	AVCTP_STATE_CONNECTED
-} avctp_state_t;
-
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 
 struct avctp_header {
@@ -148,6 +144,12 @@ struct avrcp_header {
 #error "Unknown byte order"
 #endif
 
+struct avctp_state_callback {
+	avctp_state_cb cb;
+	void *user_data;
+	unsigned int id;
+};
+
 struct avctp_server {
 	bdaddr_t src;
 	GIOChannel *io;
@@ -175,6 +177,8 @@ struct control {
 	struct avctp *session;
 };
 
+static GSList *avctp_callbacks = NULL;
+
 static sdp_record_t *avrcp_ct_record()
 {
 	sdp_list_t *svclass_id, *pfseq, *apseq, *root;
@@ -330,6 +334,7 @@ static struct avctp *avctp_get(const bdaddr_t *src, const bdaddr_t *dst)
 
 	session = g_new0(struct avctp, 1);
 
+	session->state = AVCTP_STATE_DISCONNECTED;
 	session->uinput = -1;
 	bacpy(&session->src, src);
 	bacpy(&session->dst, dst);
@@ -435,7 +440,7 @@ static void avctp_unref(struct avctp *session)
 	if (session->io_id)
 		g_source_remove(session->io_id);
 
-	if (session->dev)
+	if (session->dev && session->dev->control)
 		session->dev->control->session = NULL;
 
 	if (session->uinput >= 0) {
@@ -499,21 +504,21 @@ static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 			avrcp->code, avrcp->subunit_type, avrcp->subunit_id,
 			avrcp->opcode, operand_count);
 
-	if (avctp->packet_type == AVCTP_PACKET_SINGLE &&
-			avctp->cr == AVCTP_COMMAND &&
-			avctp->pid == htons(AV_REMOTE_SVCLASS_ID) &&
+	if (avctp->packet_type != AVCTP_PACKET_SINGLE) {
+		avctp->cr = AVCTP_RESPONSE;
+		avrcp->code = CTYPE_NOT_IMPLEMENTED;
+	} else if (avctp->pid != htons(AV_REMOTE_SVCLASS_ID)) {
+		avctp->ipid = 1;
+		avctp->cr = AVCTP_RESPONSE;
+		avrcp->code = CTYPE_REJECTED;
+	} else if (avctp->cr == AVCTP_COMMAND &&
 			avrcp->code == CTYPE_CONTROL &&
 			avrcp->subunit_type == SUBUNIT_PANEL &&
 			avrcp->opcode == OP_PASSTHROUGH) {
 		handle_panel_passthrough(session, operands, operand_count);
 		avctp->cr = AVCTP_RESPONSE;
 		avrcp->code = CTYPE_ACCEPTED;
-		ret = write(sock, buf, packet_size);
-	}
-
-	if (avctp->packet_type == AVCTP_PACKET_SINGLE &&
-			avctp->cr == AVCTP_COMMAND &&
-			avctp->pid == htons(AV_REMOTE_SVCLASS_ID) &&
+	} else if (avctp->cr == AVCTP_COMMAND &&
 			avrcp->code == CTYPE_STATUS &&
 			(avrcp->opcode == OP_UNITINFO
 			|| avrcp->opcode == OP_SUBUNITINFO)) {
@@ -521,8 +526,11 @@ static gboolean session_cb(GIOChannel *chan, GIOCondition cond,
 		avrcp->code = CTYPE_STABLE;
 		debug("reply to %s", avrcp->opcode == OP_UNITINFO ?
 				"OP_UNITINFO" : "OP_SUBUNITINFO");
-		ret = write(sock, buf, packet_size);
+	} else {
+		avctp->cr = AVCTP_RESPONSE;
+		avrcp->code = CTYPE_REJECTED;
 	}
+	ret = write(sock, buf, packet_size);
 
 	return TRUE;
 
@@ -606,6 +614,19 @@ static void init_uinput(struct avctp *session)
 		debug("AVRCP: uinput initialized for %s", address);
 }
 
+static void avctp_set_state(struct avctp *session, avctp_state_t new_state)
+{
+	GSList *l;
+	avdtp_session_state_t old_state = session->state;
+
+	session->state = new_state;
+
+	for (l = avctp_callbacks; l != NULL; l = l->next) {
+		struct avctp_state_callback *cb = l->data;
+		cb->cb(session->dev, old_state, new_state, cb->user_data);
+	}
+}
+
 static void avctp_connect_cb(GIOChannel *chan, GError *err, gpointer data)
 {
 	struct control *control = data;
@@ -619,7 +640,6 @@ static void avctp_connect_cb(GIOChannel *chan, GError *err, gpointer data)
 		debug("avctp_connect_cb: session removed while connecting");
 		if (!err)
 			g_io_channel_shutdown(chan, TRUE, NULL);
-		g_io_channel_unref(chan);
 		return;
 	}
 
@@ -656,7 +676,7 @@ static void avctp_connect_cb(GIOChannel *chan, GError *err, gpointer data)
 				AUDIO_CONTROL_INTERFACE, "Connected",
 				DBUS_TYPE_BOOLEAN, &value);
 
-	session->state = AVCTP_STATE_CONNECTED;
+	avctp_set_state(session, AVCTP_STATE_CONNECTED);
 	session->mtu = imtu;
 	session->io_id = g_io_add_watch(chan,
 				G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
@@ -707,7 +727,8 @@ static void avctp_confirm_cb(GIOChannel *chan, gpointer data)
 
 	if (!session) {
 		error("Unable to create new AVCTP session");
-		goto drop;
+		g_io_channel_shutdown(chan, TRUE, NULL);
+		return;
 	}
 
 	if (session->io) {
@@ -715,7 +736,7 @@ static void avctp_confirm_cb(GIOChannel *chan, gpointer data)
 		goto drop;
 	}
 
-	dev = manager_get_device(&src, &dst);
+	dev = manager_get_device(&src, &dst, TRUE);
 	if (!dev) {
 		error("Unable to get audio device object for %s", address);
 		goto drop;
@@ -730,9 +751,7 @@ static void avctp_confirm_cb(GIOChannel *chan, gpointer data)
 	if (!session->dev)
 		session->dev = dev;
 
-	device_remove_control_timer(dev);
-
-	session->state = AVCTP_STATE_CONNECTING;
+	avctp_set_state(session, AVCTP_STATE_CONNECTING);
 	session->io = g_io_channel_ref(chan);
 
 	if (avdtp_is_connected(&src, &dst)) {
@@ -787,10 +806,8 @@ gboolean avrcp_connect(struct audio_device *dev)
 		return FALSE;
 	}
 
-	device_remove_control_timer(dev);
-
 	session->dev = dev;
-	session->state = AVCTP_STATE_CONNECTING;
+	avctp_set_state(session, AVCTP_STATE_CONNECTING);
 
 	io = bt_io_connect(BT_IO_L2CAP, avctp_connect_cb, control, NULL, &err,
 				BT_IO_OPT_SOURCE_BDADDR, &dev->src,
@@ -804,7 +821,7 @@ gboolean avrcp_connect(struct audio_device *dev)
 		return FALSE;
 	}
 
-	g_io_channel_unref(io);
+	session->io = io;
 
 	control->session = session;
 
@@ -1005,7 +1022,7 @@ static void path_unregister(void *data)
 {
 	struct audio_device *dev = data;
 
-	info("Unregistered interface %s on path %s",
+	debug("Unregistered interface %s on path %s",
 		AUDIO_CONTROL_INTERFACE, dev->path);
 
 	control_free(dev);
@@ -1025,7 +1042,7 @@ struct control *control_init(struct audio_device *dev)
 					dev, path_unregister))
 		return NULL;
 
-	info("Registered interface %s on path %s",
+	debug("Registered interface %s on path %s",
 		AUDIO_CONTROL_INTERFACE, dev->path);
 
 	return g_new0(struct control, 1);
@@ -1041,3 +1058,34 @@ gboolean control_is_active(struct audio_device *dev)
 
 	return FALSE;
 }
+
+unsigned int avctp_add_state_cb(avctp_state_cb cb, void *user_data)
+{
+	struct avctp_state_callback *state_cb;
+	static unsigned int id = 0;
+
+	state_cb = g_new(struct avctp_state_callback, 1);
+	state_cb->cb = cb;
+	state_cb->user_data = user_data;
+	state_cb->id = ++id;
+
+	avctp_callbacks = g_slist_append(avctp_callbacks, state_cb);
+
+	return state_cb->id;
+}
+
+gboolean avctp_remove_state_cb(unsigned int id)
+{
+	GSList *l;
+
+	for (l = avctp_callbacks; l != NULL; l = l->next) {
+		struct avctp_state_callback *cb = l->data;
+		if (cb && cb->id == id) {
+			avctp_callbacks = g_slist_remove(avctp_callbacks, cb);
+			g_free(cb);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
