@@ -41,8 +41,8 @@
 #include "btio.h"
 #include "plugin.h"
 #include "logging.h"
-#include "unix.h"
 #include "device.h"
+#include "unix.h"
 #include "headset.h"
 #include "manager.h"
 
@@ -55,6 +55,8 @@ static GKeyFile *load_config_file(const char *file)
 
 	keyfile = g_key_file_new();
 
+	g_key_file_set_list_separator(keyfile, ',');
+
 	if (!g_key_file_load_from_file(keyfile, file, 0, &err)) {
 		error("Parsing %s failed: %s", file, err->message);
 		g_error_free(err);
@@ -70,7 +72,7 @@ static void sco_server_cb(GIOChannel *chan, GError *err, gpointer data)
 	int sk;
 	struct audio_device *device;
 	char addr[18];
-	bdaddr_t dst;
+	bdaddr_t src, dst;
 
 	if (err) {
 		error("sco_server_cb: %s", err->message);
@@ -78,6 +80,7 @@ static void sco_server_cb(GIOChannel *chan, GError *err, gpointer data)
 	}
 
 	bt_io_get(chan, BT_IO_SCO, &err,
+			BT_IO_OPT_SOURCE_BDADDR, &src,
 			BT_IO_OPT_DEST_BDADDR, &dst,
 			BT_IO_OPT_DEST, addr,
 			BT_IO_OPT_INVALID);
@@ -86,7 +89,8 @@ static void sco_server_cb(GIOChannel *chan, GError *err, gpointer data)
 		goto drop;
 	}
 
-	device = manager_find_device(&dst, NULL, FALSE);
+	device = manager_find_device(NULL, &src, &dst, AUDIO_HEADSET_INTERFACE,
+					FALSE);
 	if (!device)
 		goto drop;
 
@@ -103,10 +107,11 @@ static void sco_server_cb(GIOChannel *chan, GError *err, gpointer data)
 	sk = g_io_channel_unix_get_fd(chan);
 	fcntl(sk, F_SETFL, 0);
 
-	if (headset_connect_sco(device, chan) == 0) {
-		debug("Accepted SCO connection from %s", addr);
-		headset_set_state(device, HEADSET_STATE_PLAYING);
-	}
+	if (headset_connect_sco(device, chan) < 0)
+		goto drop;
+
+	debug("Accepted SCO connection from %s", addr);
+	headset_set_state(device, HEADSET_STATE_PLAYING);
 
 	return;
 
@@ -171,4 +176,4 @@ static void audio_exit(void)
 	dbus_connection_unref(connection);
 }
 
-BLUETOOTH_PLUGIN_DEFINE("audio", audio_init, audio_exit)
+BLUETOOTH_PLUGIN_DEFINE("audio", VERSION, audio_init, audio_exit)
