@@ -78,6 +78,7 @@ struct serial_port {
 	char		*uuid;		/* service identification */
 	char		*dev;		/* RFCOMM device name */
 	int		fd;		/* Opened file descriptor */
+	GIOChannel	*io;		/* BtIO channel */
 	guint		listener_id;
 	struct serial_device *device;
 };
@@ -101,13 +102,19 @@ static struct serial_device *find_device(GSList *devices, const char *path)
 static struct serial_port *find_port(GSList *ports, const char *pattern)
 {
 	GSList *l;
+	int channel;
+
+	channel = strtol(pattern, NULL, 10);
 
 	for (l = ports; l != NULL; l = l->next) {
 		struct serial_port *port = l->data;
 		char *uuid_str;
 		int ret;
 
-		if (!strcasecmp(port->uuid, pattern))
+		if (port->channel == channel)
+			return port;
+
+		if (port->uuid && !strcasecmp(port->uuid, pattern))
 			return port;
 
 		if (port->dev && !strcmp(port->dev, pattern))
@@ -118,9 +125,7 @@ static struct serial_port *find_port(GSList *ports, const char *pattern)
 			continue;
 
 		ret = strcasecmp(port->uuid, uuid_str);
-
 		g_free(uuid_str);
-
 		if (ret == 0)
 			return port;
 	}
@@ -134,6 +139,15 @@ static int port_release(struct serial_port *port)
 	int rfcomm_ctl;
 	int err = 0;
 
+	if (port->id < 0) {
+		if (port->io) {
+			g_io_channel_shutdown(port->io, TRUE, NULL);
+			g_io_channel_unref(port->io);
+			port->io = NULL;
+		}
+		return 0;
+	}
+
 	debug("Serial port %s released", port->dev);
 
 	rfcomm_ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_RFCOMM);
@@ -175,8 +189,7 @@ static void serial_port_free(struct serial_port *port)
 	if (device && port->listener_id > 0)
 		g_dbus_remove_watch(device->conn, port->listener_id);
 
-	if (port->id >= 0)
-		port_release(port);
+	port_release(port);
 
 	g_free(port->uuid);
 	g_free(port);
@@ -194,8 +207,7 @@ static void port_owner_exited(DBusConnection *conn, void *user_data)
 {
 	struct serial_port *port = user_data;
 
-	if (port->id >= 0)
-		port_release(port);
+	port_release(port);
 
 	port->listener_id = 0;
 }
@@ -324,6 +336,9 @@ static void rfcomm_connect_cb(GIOChannel *chan, GError *conn_err,
 	bacpy(&req.dst, &device->dst);
 	req.channel = port->channel;
 
+	g_io_channel_unref(port->io);
+	port->io = NULL;
+
 	sk = g_io_channel_unix_get_fd(chan);
 	port->id = ioctl(sk, RFCOMMCREATEDEV, &req);
 	if (port->id < 0) {
@@ -362,9 +377,11 @@ static void get_record_cb(sdp_list_t *recs, int err, gpointer user_data)
 	sdp_record_t *record = NULL;
 	sdp_list_t *protos;
 	DBusMessage *reply;
-	GIOChannel *io;
 	GError *gerr = NULL;
 
+	if (!port->listener_id)
+		goto failed;
+
 	if (err < 0) {
 		error("Unable to get service record: %s (%d)", strerror(-err),
 			-err);
@@ -391,21 +408,19 @@ static void get_record_cb(sdp_list_t *recs, int err, gpointer user_data)
 	sdp_list_foreach(protos, (sdp_list_func_t) sdp_list_free, NULL);
 	sdp_list_free(protos, NULL);
 
-	io = bt_io_connect(BT_IO_RFCOMM, rfcomm_connect_cb, port,
+	port->io = bt_io_connect(BT_IO_RFCOMM, rfcomm_connect_cb, port,
 				NULL, &gerr,
 				BT_IO_OPT_SOURCE_BDADDR, &device->src,
 				BT_IO_OPT_DEST_BDADDR, &device->dst,
 				BT_IO_OPT_CHANNEL, port->channel,
 				BT_IO_OPT_INVALID);
-	if (!io) {
+	if (!port->io) {
 		error("%s", gerr->message);
 		reply = failed(port->msg, gerr->message);
 		g_error_free(gerr);
 		goto failed;
 	}
 
-	g_io_channel_unref(io);
-
 	return;
 
 failed:
@@ -420,6 +435,9 @@ static int connect_port(struct serial_port *port)
 	uuid_t uuid;
 	int err;
 
+	if (!port->uuid)
+		goto connect;
+
 	err = bt_string2uuid(&uuid, port->uuid);
 	if (err < 0)
 		return err;
@@ -428,6 +446,35 @@ static int connect_port(struct serial_port *port)
 
 	return bt_search_service(&device->src, &device->dst, &uuid,
 				get_record_cb, port, NULL);
+
+connect:
+	port->io = bt_io_connect(BT_IO_RFCOMM, rfcomm_connect_cb, port,
+				NULL, NULL,
+				BT_IO_OPT_SOURCE_BDADDR, &device->src,
+				BT_IO_OPT_DEST_BDADDR, &device->dst,
+				BT_IO_OPT_CHANNEL, port->channel,
+				BT_IO_OPT_INVALID);
+	if (port->io)
+		return 0;
+
+	return -errno;
+}
+
+static struct serial_port *create_port(struct serial_device *device,
+					const char *uuid, uint8_t channel)
+{
+	struct serial_port *port;
+
+	port = g_new0(struct serial_port, 1);
+	port->uuid = g_strdup(uuid);
+	port->channel = channel;
+	port->device = device;
+	port->id = -1;
+	port->fd = -1;
+
+	device->ports = g_slist_append(device->ports, port);
+
+	return port;
 }
 
 static DBusMessage *port_connect(DBusConnection *conn,
@@ -435,16 +482,20 @@ static DBusMessage *port_connect(DBusConnection *conn,
 {
 	struct serial_device *device = user_data;
 	struct serial_port *port;
-	const char *uuid;
+	const char *pattern;
 	int err;
 
-	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &uuid,
+	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &pattern,
 						DBUS_TYPE_INVALID) == FALSE)
 		return NULL;
 
-	port = find_port(device->ports, uuid);
-	if (!port)
-		return does_not_exist(msg, "Does not match");
+	port = find_port(device->ports, pattern);
+	if (!port) {
+		int channel = strtol(pattern, NULL, 10);
+		if (channel < 1 || channel > 30)
+			return does_not_exist(msg, "Does not match");
+		port = create_port(device, NULL, channel);
+	}
 
 	if (port->listener_id)
 		return failed(msg, "Port already in use");
@@ -492,8 +543,7 @@ static DBusMessage *port_disconnect(DBusConnection *conn,
 	if (!g_str_equal(owner, caller))
 		return failed(msg, "Operation not permited");
 
-	if (port->id >= 0)
-		port_release(port);
+	port_release(port);
 
 	g_dbus_remove_watch(conn, port->listener_id);
 	port->listener_id = 0;
